#Modified from the chapter on virtual power
#ImportFEniCSlibrary
from fenics import *
#Parameters
h=2.        #height of the layer
s12bc=.5	#applied shear stress at boundary
s22bc=.25	#applied normal stress at boundary
Eel=1000.	#Young modulus
nuel=0.		#Poisson coefficient
ny=3		#number of elements-FE discretization
Gel=Eel/(2.*(1+nuel))
Mel=Eel*(1.-nuel)/((1.-2.*nuel)*(1.+nuel))

#Generate mesh
mesh=IntervalMesh(ny,0.,h/2.)
#Define element topology (1D)
cell=interval
#Defines a Lagrangian FE of degree 1 and two unknowns in each node(vector)
element=VectorElement("Lagrange",cell,degree=1,dim=2)
#Assign the element to the mesh
V=FunctionSpace(mesh,element)

#Define test function (virtual velocities)
v=TestFunction(V)
#Define trial (unknown) function
u=TrialFunction(V)
#Store the solution to sol
sol=Function(V)

#Define boundary conditions
def middle(x, on_boundary):
	return on_boundary and near(x[0],0.)
bc=DirichletBC(V,(0.,0.),middle)
#Define traction vector
ti=Constant((s22bc,s12bc))

#Define internal virtual power
Pint=(
 Mel*Dx(u[0],0)*Dx(v[0],0)+	#sigma22*v2,2
 Gel*Dx(u[1],0)*Dx(v[1],0)	#sigma12*v1,2
 )*dx
#Define external virtual power
Pext=dot(ti,v)*ds
#Solve the problem (does the FE formulation, matrix assembly and linear solve)
solve(Pint==Pext,sol,bc)

#Plot solution
import matplotlib.pyplot as plt
plot(sol[1],label='$u_x$')
plot(sol[0],label='$u_y$')
plt.xlabel("$y$")
plt.legend(loc='upper left')
plt.ylim(0,.001)
font={'size':18}
plt.rc('font',**font)
plt.show()

# evaluating the stored energy
energy = 0.5*(
 Mel*Dx(sol[0],0)*Dx(sol[0],0)+	
 Gel*Dx(sol[1],0)*Dx(sol[1],0)	
 )*dx
E = assemble(energy)	

print('energy:',
      E)