import matplotlib.pyplot as plt
import numpy as np

# Some useful parameters
fmt='%1.8f'
prec=1e-9
# Units are in SI
# Material parameters
# Elastic
nu = 0.2
E  = 2000000.0
lamda = E * nu / ((1.0 + nu) * (1.0 - 2.0 * nu))
mu    = E / (2.0 * (1.0 + nu))
# Plastic
cu =20000.
phi= np.pi*20./180.
alpha=-np.pi+49.0*2.0*np.pi/49
alpha= 10.0/180.*np.pi
af =np.sin(phi)
ag =np.sin(phi+alpha)
mult =(mu+(mu+lamda)*np.sin(phi)*np.sin(phi+alpha))

# Macaulay bracket
ppos = lambda x: (x+abs(x))/2.

# Heaviside
pone = lambda x: (1+x/(abs(x)+1e-8))/2.

# Units are in SI
# Concerning the time
Tf = 1.0            # final time
num_steps = 200000 	# number of time steps
dt = Tf / (num_steps-1) # time step size
# Time-stepping
t = 0
# Vertical strain increment
dudisp= 0.15*dt/Tf

#Preparing for the results
#Total strains
e11 = np.zeros(num_steps)
e22 = np.zeros(num_steps)
#Plastic strains
e11p= np.zeros(num_steps)
e22p= np.zeros(num_steps)
#Elastic strains
e11e= np.zeros(num_steps)
e22e= np.zeros(num_steps)
#Stresses
s11= np.zeros(num_steps)
s22= np.zeros(num_steps)
#Energies - elastic and plastic
Eel= np.zeros(num_steps)
Epl= np.zeros(num_steps)
Ein= np.zeros(num_steps)
Eut= np.zeros(num_steps)

lam= np.zeros(num_steps)
tst= np.zeros(num_steps)

#Initial stress state
p0 = 50000.
s11[0]= p0
s22[0]= p0

lin = np.zeros(num_steps)

for i in range(1,num_steps):
    lin[i]=-0.5+i*1.0/(num_steps-1)
# The vertical strain increment is known
    e22[i]=e22[i-1]+dudisp
# The horizontal stress is known
    s11[i]=p0	
# Assuming the step is elastic	
    e11[i]= e11[i-1]-lamda*dudisp/(2.*mu +lamda)
    s22[i]= s22[i-1]+(2.*mu+lamda)*(e22[i]-e22[i-1])+lamda*(e11[i]-e11[i-1])
    f_y=(s22[i]-s11[i])/2.-af*(s22[i]+s11[i])/2.-cu
    ll = ppos(f_y)
# If the step is plastic
    if ll>prec:
        ll = ll/mult
        lam[i] = ll
# the plastic strain increments are known
        e11p[i]= e11p[i-1]-ll*(1.+ag)
        e22p[i]= e22p[i-1]+ll*(1.-ag)  
# the vertical strain is known
        e22e[i]= e22e[i-1]+dudisp-(e22p[i]-e22p[i-1])  	
# the horizontal stress is known
        s11[i]=p0
# the vertical stress is known		
        s22[i]=s22[i-1]+E*(e22e[i]-e22e[i-1])+nu*(s11[i]-s11[i-1])
# the elastic strains are known
        e11e[i]= e11e[i-1]+(s11[i]-s11[i-1]-nu*(s22[i]-s22[i-1]))/E
        e22[i]=e22[i-1]+e22e[i]-e22e[i-1]+e22p[i]-e22p[i-1]
        e11[i]=e11[i-1]+e11e[i]-e11e[i-1]+e11p[i]-e11p[i-1]	
        tst[i]=np.arcsin((e22p[i]-e11p[i])/(e22p[i]+e11p[i]))*180/np.pi	
    else:
        e11e[i]= e11e[i-1]+e11[i]-e11[i-1]
        e22e[i]= e22e[i-1]+e22[i]-e22[i-1]
        e11p[i]= e11p[i-1]
        e22p[i]= e22p[i-1]		
    Eel[i]=Eel[i-1]+s11[i]*(e11e[i]-e11e[i-1])+s22[i]*(e22e[i]-e22e[i-1])  
    Epl[i]=Epl[i-1]+s11[i]*(e11p[i]-e11p[i-1])+s22[i]*(e22p[i]-e22p[i-1])    	
    Ein[i]=Ein[i-1]+s22[i]*(e22[i]-e22[i-1])+s11[i]*(e11[i]-e11[i-1]) 
    Eut[i]=Eut[i-1]+s11[i]*(e11p[i]-e11p[i-1])  	

fmt='%1.8f'

p=(s22+s11)/2.0
q=(s22-s11)/2.0
plt.plot(p,q,label='pq')
plt.xlabel("$p$")
plt.ylabel("$q$")
plt.legend(loc='upper left')
#plt.ylim(0,.0001)
font={'size':20}
plt.rc('font',**font)
plt.show()

p=(e22p+e11p)/2.0
q=(e22p-e11p)/2.0
plt.plot(p,q,label='pq')
plt.plot(lin,lin*af,label='test')
plt.xlabel("$ep$")
plt.ylabel("$eq$")
plt.legend(loc='upper left')
plt.xlim(-0.5,0.5)
plt.ylim(-0.5,0.5)
font={'size':20}
plt.rc('font',**font)
plt.show()

plt.plot(e22,tst,label='pq')
plt.xlabel("$ep$")
plt.ylabel("$tst$")
font={'size':20}
plt.rc('font',**font)
plt.show()

plt.plot(e22,s22,label='yy')
plt.xlabel("$\epsilon_{22}$")
plt.ylabel("$\sigma_{22}$")
plt.legend(loc='upper left')
#plt.ylim(0,.0001)
font={'size':20}
plt.rc('font',**font)
plt.show()

plt.plot(e22,lam,label='ll')
plt.xlabel("$\epsilon_{22}$")
plt.ylabel("$\lambda$")
plt.legend(loc='upper left')
#plt.ylim(0,.0001)
font={'size':20}
plt.rc('font',**font)
plt.show()

plt.plot(e11p,e22p,label='yy')
plt.xlabel("$\epsilon_{11}^{pl}$")
plt.ylabel("$\epsilon_{22}^{pl}$")
plt.legend(loc='upper left')
#plt.ylim(0,.0001)
font={'size':20}
plt.rc('font',**font)
plt.show()

plt.plot(e22,Eel,label='el')
plt.plot(e22,Epl,label='pl')
plt.plot(e22,Ein,label='tot')
plt.plot(e22,Eut,label='11pl')
#plt.plot(e22,Ein-Epl,label='out')
plt.xlabel("$\epsilon_{22}$")
plt.ylabel("$Energy}$")
plt.legend(loc='upper left')
#plt.ylim(0,.0001)
font={'size':20}
plt.rc('font',**font)
plt.show()

