import matplotlib.pyplot as plt
import numpy as np

# Some useful parameters
fmt='%1.8f'
prec=1e-9
# Units are in SI
# Material parameters
# Elastic
nu = 0.2
E  = 2000000.0
lamda = E * nu / ((1.0 + nu) * (1.0 - 2.0 * nu))
mu    = E / (2.0 * (1.0 + nu))
# Plastic
cu =20.
phi= 1.*np.pi/9.
alpha=1.*np.pi
af =np.sin(phi)
ag =np.sin(phi+alpha)
mult =(mu+(mu+lamda)*np.sin(phi)*np.sin(phi+alpha))

# Macaulay bracket
ppos = lambda x: (x+abs(x))/2.

# Heaviside
pone = lambda x: (1+x/(abs(x)+1e-8))/2.

# Units are in SI
# Concerning the time
Tf = 1.0            # final time
num_steps =  200000 	  	# number of time steps
dt = Tf / (num_steps-1) # time step size
# Time-stepping
t = 0
# Vertical strain increment
dudisp= 0.1*dt/Tf

#Preparing for the results
#Total strains
e11 = np.zeros(num_steps)
e22 = np.zeros(num_steps)
#Plastic strains
e11p= np.zeros(num_steps)
e22p= np.zeros(num_steps)
#Elastic strains
e11e= np.zeros(num_steps)
e22e= np.zeros(num_steps)
#Stresses
s11= np.zeros(num_steps)
s22= np.zeros(num_steps)

na = 100
aa = np.zeros(na)
da = 2.*np.pi/(na-1.)

#Energies - elastic and plastic
Eel= np.zeros(na)
Epl= np.zeros(na)
Ein= np.zeros(na)
Eut= np.zeros(na)

for j in range(na):
    print('iteration:',
          j)
    #Initial stress state
    p0 = 50000.
    s11[0]= p0
    s22[0]= p0
	
    aa[j] = -np.pi+j*da
    alpha=aa[j]
    ag =np.sin(phi+alpha)
    mult =(mu+(mu+lamda)*np.sin(phi)*np.sin(phi+alpha))	
    for i in range(1,num_steps):
    # The vertical strain increment is known
        e22[i]=e22[i-1]+dudisp
    # The horizontal stress is known
        s11[i]=p0	
    # Assuming the step is elastic	
        e11[i]= e11[i-1]-lamda*dudisp/(2.*mu +lamda)
        s22[i]= s22[i-1]+(2.*mu+lamda)*(e22[i]-e22[i-1])+lamda*(e11[i]-e11[i-1])
        f_y=(s22[i]-s11[i])/2.-af*(s22[i]+s11[i])/2.-cu
        ll = ppos(f_y)
    # If the step is plastic
        if ll>prec:
            ll = ll/mult#*(np.sign(np.cos(phi+alpha))+1e-8)
    # the plastic strain increments are known
            e11p[i]= e11p[i-1]-ll*(1.+ag)
            e22p[i]= e22p[i-1]+ll*(1.-ag)   
    # the vertical strain is known
            e22e[i]= e22e[i-1]+dudisp-(e22p[i]-e22p[i-1])  	
    # the horizontal stress is known
            s11[i]=p0
    # the vertical stress is known		
            s22[i]=s22[i-1]+E*(e22e[i]-e22e[i-1])+nu*(s11[i]-s11[i-1])
    # the elastic strains are known
            e11e[i]= e11e[i-1]+(s11[i]-s11[i-1]-nu*(s22[i]-s22[i-1]))/E
            e22[i]=e22[i-1]+e22e[i]-e22e[i-1]+e22p[i]-e22p[i-1]
            e11[i]=e11[i-1]+e11e[i]-e11e[i-1]+e11p[i]-e11p[i-1]			
        else:
            e11e[i]= e11e[i-1]+e11[i]-e11[i-1]
            e22e[i]= e22e[i-1]+e22[i]-e22[i-1]
            e11p[i]= e11p[i-1]
            e22p[i]= e22p[i-1]		
        Eel[j]=Eel[j]+s11[i]*(e11e[i]-e11e[i-1])+s22[i]*(e22e[i]-e22e[i-1])  
        Epl[j]=Epl[j]+s11[i]*(e11p[i]-e11p[i-1])+s22[i]*(e22p[i]-e22p[i-1])    	
        Ein[j]=Ein[j]+s22[i]*(e22[i]-e22[i-1])+s11[i]*(e11[i]-e11[i-1]) 
        Eut[j]=Eut[j]+s11[i]*(e11p[i]-e11p[i-1])   	

fmt='%1.8f'
aa = aa*180./np.pi
plt.plot(aa,Eel,label='el')
plt.plot(aa,Epl,label='pl')
plt.plot(aa,Ein,label='tot')
plt.plot(aa,Eut,label='11pl')
plt.xlabel("$a$")
plt.ylabel("$Energy}$")
plt.legend(loc='upper left')
plt.ylim(-10000,10000)
font={'size':20}
plt.rc('font',**font)
plt.show()

plt.plot(aa,Eel,label='el')
plt.plot(aa,Epl,label='pl')
plt.plot(aa,Ein,label='tot')
#plt.plot(aa,Eut,label='11pl')
#plt.plot(e22,Ein-Epl,label='out')
plt.xlabel("$a$")
plt.ylabel("$Energy}$")
plt.legend(loc='upper left')
#plt.ylim(0,.0001)
font={'size':20}
plt.rc('font',**font)
plt.show()

plt.plot(aa,np.sign(Epl),label='pl')
plt.xlabel("$a$")
plt.ylabel("$Plastic Energy}$")
plt.legend(loc='upper left')
font={'size':20}
plt.rc('font',**font)
plt.show()
